package com.yeejoin.amos.boot.module.tzs.biz.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yeejoin.amos.boot.biz.common.controller.BaseController;
import com.yeejoin.amos.boot.module.tzs.api.dto.EnevatorRelationDto;
import com.yeejoin.amos.boot.module.tzs.biz.service.impl.EnevatorRelationServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.typroject.tyboot.core.foundation.enumeration.UserType;
import org.typroject.tyboot.core.restful.doc.TycloudOperation;
import org.typroject.tyboot.core.restful.utils.ResponseHelper;
import org.typroject.tyboot.core.restful.utils.ResponseModel;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * 关注电梯关系表
 *
 * @author litw
 * @date 2021-09-22
 */
@RestController
@Api(tags = "关注电梯关系表Api")
@RequestMapping(value = "/enevator-relation")
public class EnevatorRelationController extends BaseController {

    @Autowired
    EnevatorRelationServiceImpl enevatorRelationServiceImpl;

    /**
     * 新增关注电梯关系表
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
	@PostMapping(value = "/save")
	@ApiOperation(httpMethod = "POST", value = "新增关注电梯关系表", notes = "新增关注电梯关系表")
	public ResponseModel<EnevatorRelationDto> save(@RequestBody EnevatorRelationDto model) {
	    model = enevatorRelationServiceImpl.createWithModel(model);
	    return ResponseHelper.buildResponse(model);
	}

	/**
     * 根据sequenceNbr删除 -- 取消关注
     *
     * @param sequenceNbr 主键
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @DeleteMapping(value = "/{openId}/{enevatorId}")
    @ApiOperation(httpMethod = "DELETE", value = "根据sequenceNbr删除关注电梯关系表", notes = "根据sequenceNbr删除关注电梯关系表")
    public ResponseModel<Boolean> deleteBySequenceNbr(HttpServletRequest request, @PathVariable(value = "openId") Long openId,
													  @PathVariable(value = "enevatorId") String sequenceNbr){
        return ResponseHelper.buildResponse(enevatorRelationServiceImpl.removeById(sequenceNbr));
    }

    /**
     * 根据sequenceNbr查询
     *
	 * @param sequenceNbr  主键
     * @return
     */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/{sequenceNbr}")
	@ApiOperation(httpMethod = "GET",value = "根据sequenceNbr查询单个关注电梯关系表",  notes = "根据sequenceNbr查询单个关注电梯关系表")
	public ResponseModel<EnevatorRelationDto> selectOne(@PathVariable Long sequenceNbr) {
	    return ResponseHelper.buildResponse(enevatorRelationServiceImpl.queryBySeq(sequenceNbr));
	}

	/**
     * 列表分页查询
     *
     * @param current 当前页
     * @param current 每页大小
     * @return
     */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/page")
	@ApiOperation(httpMethod = "GET",value = "关注电梯关系表分页查询",  notes = "关注电梯关系表分页查询")
	public ResponseModel<Page<EnevatorRelationDto>> queryForPage(@RequestParam(value = "current") int current,@RequestParam
                    (value = "size") int size) {
	    Page<EnevatorRelationDto> page = new Page<EnevatorRelationDto>();
	    page.setCurrent(current);
	    page.setSize(size);
	    return ResponseHelper.buildResponse(enevatorRelationServiceImpl.queryForEnevatorRelationPage(page));
	}

	/**
	 * 列表全部数据查询
     *
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation(httpMethod = "GET",value = "关注电梯关系表列表全部数据查询",   notes = "关注电梯关系表列表全部数据查询")
	@GetMapping(value = "/list")
	public ResponseModel<List<EnevatorRelationDto>> selectForList() {
	    return ResponseHelper.buildResponse(enevatorRelationServiceImpl.queryForEnevatorRelationList());
	}
}
